/* Nessus
 * Copyright (C) 1998, 1999, 2000 Renaud Deraison
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */  

#include <includes.h>

#ifdef USE_GTK
#include <gtk/gtk.h>
#include "xstuff.h"
#include "xpm/yellow.xpm"
#include "xpm/red.xpm"
#include "xpm/white.xpm"
#include "html_graph_output.h"
#endif

#include "report.h"
#include "families.h"
#include "nsr_output.h"
#include "html_output.h"
#include "report_utils.h"
#include "error_dialog.h"
#include "latex_output.h"
#include "text_output.h"
#include "globals.h"


#define SAVE_NSR 0
#define SAVE_HTML 1
#define SAVE_TEXT 2
#define SAVE_LATEX 3
#define SAVE_HTML_GRAPH 4
#define SAVE_MAX SAVE_HTML_GRAPH
 
#ifdef USE_GTK
static void save_report_ask(GtkWidget *, GtkWidget *);
static void save_report(GtkWidget *, GtkWidget *);
GtkWidget * report_to_tree(struct arglist *, GtkWidget *);
#endif



#ifdef USE_GTK

static void 
report_click(GtkWidget * list, GtkWidget * data)
{
 GtkWidget * window = gtk_object_get_data(GTK_OBJECT(list), "window");
 GtkWidget * box = gtk_object_get_data(GTK_OBJECT(list), "box");
 GList * dlist = GTK_LIST(list)->selection;
 GtkObject * item;
 struct arglist * report;
 GtkWidget * tree = NULL;
 GtkWidget * old_tree = NULL;
 int new_tree = 0;
 if(!dlist)
  return;
  
 item = GTK_OBJECT(dlist->data);
 report = gtk_object_get_data(item, "results");
 
 if(report){
 	tree = gtk_object_get_data(item, "tree");
	if(!tree)
	{
 	 tree = report_to_tree(report->value, window);
	 gtk_object_set_data(item, "tree", tree);
	 new_tree++;
	}
    }
    
    			
 if(tree)
 {
   old_tree = gtk_object_get_data(GTK_OBJECT(box), "tree");
   if(old_tree){
   	GtkAdjustment * adj;
   	gtk_widget_hide(old_tree);
	adj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(window));
   	gtk_adjustment_set_value(adj, 0);
	}
    if(new_tree)
    {
     gtk_box_pack_start(GTK_BOX(box), tree, TRUE, TRUE, 0);
    } 
     gtk_widget_show(tree);
#if GTK_VERSION > 10
    gtk_widget_map(tree);
#endif  
  
  gtk_widget_realize(tree);
  gtk_object_set_data(GTK_OBJECT(box), "tree", tree);
 }
}




static
GtkWidget * summary_host_label(name, window, severity)
 char * name;
 GtkWidget * window;
 int severity;
{
 GtkWidget * label;
 GtkWidget * hbox;
 char ** pixdata = NULL;
 GdkPixmap * pixmap;
 GtkStyle * style;
 GdkBitmap * mask;
 GtkWidget * pixmapwid;
 switch(severity)
 {
  case HOLE_PRESENT :
  	pixdata = red_dot_xpm;
	break;
  case WARNING_PRESENT :
  	pixdata = yellow_dot_xpm;
	break;
  default :
  	pixdata = white_dot_xpm;
	break;
 }
 style = gtk_widget_get_style(window);
 pixmap = gdk_pixmap_create_from_xpm_d(window->window, &mask,
            &style->bg[GTK_STATE_NORMAL],(gchar **)pixdata); 
            
 pixmapwid = gtk_pixmap_new(pixmap, mask);
 hbox = gtk_hbox_new(FALSE,FALSE);
               
 label = gtk_label_new(name);
 gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
 gtk_box_pack_start(GTK_BOX(hbox), pixmapwid, FALSE, FALSE, 0);
 gtk_widget_show(pixmapwid);
 gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
 gtk_widget_show(label);
 return hbox;
}
static void 
fill_summary_box(box, scrolled, results)
 GtkWidget * box, * scrolled;
 struct arglist * results;
{
 GtkWidget * label = gtk_label_new("Summary");
 GtkWidget * sep = gtk_hseparator_new();
 char * data = emalloc(4096);
 GtkWidget * window;
 GtkWidget * list;
 GtkWidget * hbox;
 
 GList * dlist = NULL;
 
 gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 0);
 gtk_widget_show(label);
 
 gtk_box_pack_start(GTK_BOX(box), sep, FALSE, FALSE, 0);
 gtk_widget_show(sep);
 
 sprintf(data, "Number of hosts tested : %d", arglist_length(results));
 label = gtk_label_new(data);
 gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 0);
 gtk_widget_show(label);
 
 sprintf(data, "Found %d security holes", number_of_holes(results));
 label = gtk_label_new(data);
 gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 0);
 gtk_widget_show(label);
 
 sprintf(data, "Found %d security warnings", number_of_warnings(results));
 label = gtk_label_new(data);
 gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 0);
 gtk_widget_show(label);
 
 sep = gtk_hseparator_new();
 gtk_box_pack_start(GTK_BOX(box), sep, FALSE, FALSE, 0);
 gtk_widget_show(sep);
 
 window = gtk_scrolled_window_new(NULL,NULL);

 gtk_box_pack_start(GTK_BOX(box), window, TRUE, TRUE, 0);

 
 list = gtk_list_new();
#if GTK_VERSION < 11
  gtk_container_add(GTK_CONTAINER(window),list);
#else
  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(window), list);
#endif
 gtk_widget_show(list);
 gtk_object_set_data(GTK_OBJECT(list), "window", scrolled);
 hbox = gtk_hbox_new(FALSE, FALSE);
#if GTK_VERSION < 11
  gtk_container_add(GTK_CONTAINER(scrolled),hbox);
#else
  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled), hbox);
#endif 
  gtk_object_set_data(GTK_OBJECT(list), "box", hbox);
 gtk_widget_show(hbox);
 gtk_widget_realize(scrolled);
  gtk_signal_connect(GTK_OBJECT(list),
		     "selection_changed",
		     GTK_SIGNAL_FUNC(report_click),
		     NULL);
		     
		     
 while(results && results->next)
 {
  GtkWidget * item;
  int severity;
  
  
  item = gtk_list_item_new();
  gtk_object_set_data(GTK_OBJECT(item), "results", results->value);
  severity =  number_of_holes_by_host(results->value);
  if(severity)severity = HOLE_PRESENT;
  else {
  	severity =  number_of_warnings_by_host(results->value);
 	if(severity)severity = WARNING_PRESENT;
	}		 
  label = summary_host_label(results->name, scrolled, severity);
  gtk_container_add(GTK_CONTAINER(item), label);
  gtk_widget_show(label);
  dlist = g_list_append(dlist, item);
  gtk_widget_show(item);
  results = results->next;
 }
 gtk_list_append_items(GTK_LIST(list), dlist);
 gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(window), 
  		 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC); 

 gtk_widget_show(window);
 gtk_widget_realize(window);
 free(data);
}

/*
 * Creation of the reporting window
 */
static void create_report_window(args)
  struct arglist * args;
{
  
  GtkWidget * window;
  GtkWidget * widget;
  GtkWidget * vbox;
  GtkWidget * hbox;
  GtkWidget * paned;
  
  GtkWidget * button;
  GtkWidget * optionmenu;
  GtkWidget * type;
  GtkWidget * menu;
  GtkWidget * summary_box;
  struct arglist * report = args;
  struct arglist * hosts;
  
  if(!args || !args->next){
  	char * str = emalloc(255);
	sprintf(str, "No problem has been found, or none of the \
hosts tested was alive"); 
  	show_error(str);
	free(str);
	return;
	}
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_usize(window, 640,480);
  gtk_signal_connect(GTK_OBJECT(window), "destroy", 
     	GTK_SIGNAL_FUNC(close_window), window);
  gtk_signal_connect(GTK_OBJECT(window), "delete_event", 
  	GTK_SIGNAL_FUNC(delete_event), window);
       
  gtk_window_set_title(GTK_WINDOW(window), "Nessus Report");
  gtk_container_border_width(GTK_CONTAINER(window), 10);
  
  paned = gtk_hpaned_new();
  gtk_paned_set_gutter_size(GTK_PANED(paned), 15);
  gtk_container_add(GTK_CONTAINER(window), paned);
  gtk_widget_show(paned);
  
  vbox = gtk_vbox_new(FALSE, 0);
 
   summary_box = gtk_vbox_new(FALSE, 5);
  
  gtk_paned_add1(GTK_PANED(paned), summary_box);
  gtk_paned_add2(GTK_PANED(paned), vbox);
  

  
  
  widget = gtk_scrolled_window_new(NULL,NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(widget), 
  		 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 
  gtk_box_pack_start(GTK_BOX(vbox), widget, TRUE, TRUE, 5);
  gtk_widget_show(widget);
  
  fill_summary_box(summary_box, widget, args);
  gtk_widget_show(summary_box);
  
  
  hosts = emalloc(sizeof(struct arglist));
 
 #if 0
  while(args && args->next)
  {
   struct arglist * ports = arg_get_value(args->value, "PORTS");
   struct arglist * new_ports = emalloc(sizeof(struct arglist));
   
 
   while(ports && ports->next)
   {
    struct arglist * reports = arg_get_value(ports->value, "REPORT");
    struct arglist * infos = arg_get_value(ports->value, "INFO");
    struct arglist * data = emalloc(sizeof(struct arglist));
    
     
    if(reports || infos)
    {
     if(reports)arg_add_value(data, "Security holes", ARG_ARGLIST, -1, reports);
     if(infos)arg_add_value(data, "Security warnings", ARG_ARGLIST, -1, infos);
    }
    arg_add_value(new_ports, ports->name, ARG_ARGLIST, -1, data);
    ports = ports->next;
   }
   arg_add_value(hosts, args->name, ARG_ARGLIST, -1, new_ports);
   args = args->next;
  }
 #endif 
  gtk_widget_realize(widget);
  
 
 
  hbox = gtk_hbox_new(TRUE,10);

  button = gtk_button_new_with_label("Save as...");
  
  gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
  gtk_widget_show(button);
  
  optionmenu = gtk_option_menu_new();
  menu = gtk_menu_new();
  gtk_option_menu_set_menu(GTK_OPTION_MENU(optionmenu), menu);
  gtk_widget_show(menu);
  
  gtk_object_set_data(GTK_OBJECT(menu), "hosts", report);
  
  type = gtk_menu_item_new_with_label("Save as NSR");
  gtk_object_set_data(GTK_OBJECT(type), "type", (void*)SAVE_NSR);
  gtk_menu_append(GTK_MENU(menu), type);
  gtk_widget_show(type);
  
  type = gtk_menu_item_new_with_label("Save as HTML");
  gtk_object_set_data(GTK_OBJECT(type), "type", (void*)SAVE_HTML);
  gtk_menu_append(GTK_MENU(menu), type);
  gtk_widget_show(type);
  
  type = gtk_menu_item_new_with_label("Save as LaTeX");
  gtk_object_set_data(GTK_OBJECT(type), "type", (void*)SAVE_LATEX);
  gtk_menu_append(GTK_MENU(menu), type);
  gtk_widget_show(type);
  
  type = gtk_menu_item_new_with_label("Save as ASCII text");
  gtk_object_set_data(GTK_OBJECT(type), "type", (void*)SAVE_TEXT);
  gtk_menu_append(GTK_MENU(menu), type);
  gtk_widget_show(type);
 

  type = gtk_menu_item_new_with_label("Save as HTML with Pies and Graphs");
  gtk_object_set_data(GTK_OBJECT(type), "type", (void*)SAVE_HTML_GRAPH);
  gtk_menu_append(GTK_MENU(menu), type);
  gtk_widget_show(type);
  
  gtk_signal_connect(GTK_OBJECT(button),"clicked",
                    GTK_SIGNAL_FUNC(save_report_ask),menu); 
  gtk_box_pack_start(GTK_BOX(hbox), optionmenu, TRUE, TRUE, 0);
  gtk_widget_show(optionmenu);

  button = gtk_button_new_with_label("Close");
  gtk_signal_connect(GTK_OBJECT(button),"clicked",
                    GTK_SIGNAL_FUNC(close_window),(void *) window); 
  gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
  gtk_widget_show(button);
  
  gtk_box_pack_end(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
 
  gtk_widget_show(hbox);
  gtk_widget_show(vbox);

  gtk_widget_show(window);
}



static
GtkWidget * data_to_tree_build_label(name)
 char * name;
{
 GtkWidget * label;
 GtkWidget * hbox;
 
 hbox = gtk_hbox_new(FALSE,FALSE);
               
 label = gtk_label_new(name);
 gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
 gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 15);
 gtk_widget_show(label);
 return hbox;
}

static 
GtkWidget * data_to_tree(arglist)
 struct arglist * arglist;
{
 GtkWidget * tree;
 
 if(!(arglist && arglist->next))
  return NULL;
 
 tree = gtk_tree_new();
 
 while(arglist && arglist->next)
 {
  GtkWidget * tree_item;
  GtkWidget * report_data = data_to_tree_build_label(arglist->value);
  
  tree_item = gtk_tree_item_new();
 
  gtk_container_add(GTK_CONTAINER(tree_item), report_data);
  gtk_widget_show(report_data);
  
  gtk_tree_append(GTK_TREE(tree), tree_item);
  gtk_tree_item_expand(GTK_TREE_ITEM(tree_item));
  gtk_tree_item_collapse(GTK_TREE_ITEM(tree_item));
 
  gtk_widget_show(tree_item);
  arglist = arglist->next;
  }
 return tree;
}


static
GtkWidget * infos_and_holes_to_tree_build_label(name, severity)
 char * name;
 int severity;
{
 GtkWidget * label;
 GtkWidget * hbox;
 
 hbox = gtk_hbox_new(FALSE,FALSE);
               
 label = gtk_label_new(name);
 gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
 gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 15);
 gtk_widget_show(label);
 return hbox;
}



static
GtkWidget * infos_and_holes_to_tree(arglist, window)
 struct arglist * arglist;
 GtkWidget * window;
{
 int level = is_there_any_hole(arglist);
 GtkWidget * tree;
 GtkWidget * tree_item;
 GtkWidget * label;
 if(!level)
  return NULL;
  

 tree = gtk_tree_new();
 if(arg_get_value(arglist, "INFO"))
 {
  GtkWidget * subtree;
  tree_item = gtk_tree_item_new();
  label = infos_and_holes_to_tree_build_label("Security warnings", 1);
  gtk_container_add(GTK_CONTAINER(tree_item), label);
  gtk_widget_show(label);
  gtk_tree_append(GTK_TREE(tree), tree_item);
  subtree = data_to_tree(arg_get_value(arglist, "INFO"));
  if(subtree)
  {
   gtk_tree_item_set_subtree(GTK_TREE_ITEM(tree_item), subtree);
   gtk_widget_show(subtree);
  }
  gtk_tree_item_expand(GTK_TREE_ITEM(tree_item));
  gtk_tree_item_collapse(GTK_TREE_ITEM(tree_item));
 
  gtk_widget_show(tree_item);
 }
 
 if(arg_get_value(arglist, "REPORT"))
 {
  GtkWidget * subtree;
  tree_item = gtk_tree_item_new();
  label = infos_and_holes_to_tree_build_label("Security holes", 2);
  gtk_container_add(GTK_CONTAINER(tree_item), label);
  gtk_widget_show(label);
  gtk_tree_append(GTK_TREE(tree), tree_item);
  subtree = data_to_tree(arg_get_value(arglist, "REPORT"));
  if(subtree)
  {
   gtk_tree_item_set_subtree(GTK_TREE_ITEM(tree_item), subtree);
   gtk_widget_show(subtree);
  }
  gtk_tree_item_expand(GTK_TREE_ITEM(tree_item));
  gtk_tree_item_collapse(GTK_TREE_ITEM(tree_item));
  gtk_widget_show(tree_item);
 }
 
 return tree;  
}

static
GtkWidget * report_build_label(name, window, severity)
 char * name;
 GtkWidget * window;
 int severity;
{
 GtkWidget * label;
 GtkWidget * hbox;
 char ** pixdata = NULL;
 GdkPixmap * pixmap;
 GtkStyle * style;
 GdkBitmap * mask;
 GtkWidget * pixmapwid;
 switch(severity)
 {
  case HOLE_PRESENT :
  	pixdata = red_dot_xpm;
	break;
  case WARNING_PRESENT :
  	pixdata = yellow_dot_xpm;
	break;
  default :
  	pixdata = white_dot_xpm;
	break;
 }
 style = gtk_widget_get_style(window);
 pixmap = gdk_pixmap_create_from_xpm_d(window->window, &mask,
            &style->bg[GTK_STATE_NORMAL],(gchar **)pixdata); 
            
 pixmapwid = gtk_pixmap_new(pixmap, mask);
 hbox = gtk_hbox_new(FALSE,FALSE);
               
 label = gtk_label_new(name);
 gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
 gtk_box_pack_start(GTK_BOX(hbox), pixmapwid, FALSE, FALSE, 0);
 gtk_widget_show(pixmapwid);
 gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
 gtk_widget_show(label);
 return hbox;
}


GtkWidget * report_to_tree(arglist, window)
  struct arglist * arglist;
  GtkWidget * window;
{
 GtkWidget * tree;
 GtkWidget * tree_item;
 int content;
 
 
 tree = gtk_tree_new();
 
 
 
 while(arglist && arglist->next)
 {
  GtkWidget * tree_item;
  GtkWidget * label;
  GtkWidget * subtree;
  
  
  tree_item = gtk_tree_item_new();
  
   /*
    * Make label here
    */
    
   label = report_build_label(arglist->name, window, is_there_any_hole(arglist->value));
   gtk_container_add(GTK_CONTAINER(tree_item), label);
   gtk_widget_show(label);
   gtk_tree_append(GTK_TREE(tree), tree_item);
   
   
   if((subtree = infos_and_holes_to_tree(arglist->value, window)))
   {
    gtk_tree_item_set_subtree(GTK_TREE_ITEM(tree_item), subtree);
    gtk_widget_show(subtree);
   }
   gtk_tree_item_expand(GTK_TREE_ITEM(tree_item));
   gtk_tree_item_collapse(GTK_TREE_ITEM(tree_item));
 
   gtk_widget_show(tree_item);
   arglist = arglist->next;
 }
  return(tree);
}    
#endif
int is_there_any_hole(arglist)
 struct arglist * arglist;
{
 int ret = 0;
 while(arglist && arglist->next && (ret!=HOLE_PRESENT))
 {
  int tmp = 0;
  if(!strcmp(arglist->name, "Security warnings")||
     !strcmp(arglist->name, "INFO"))ret = WARNING_PRESENT;
  else if(!strcmp(arglist->name, "Security holes")||
     !strcmp(arglist->name, "REPORT"))ret = HOLE_PRESENT;
  /*
   * Check in the sublist
   */
  if(arglist->type == ARG_ARGLIST)tmp = is_there_any_hole(arglist->value);
  if(tmp >= ret)ret = tmp;
  arglist = arglist->next;
 }
 return(ret);
}

#ifdef USE_GTK
/*
 *  Main function of the holes reporter
 *  Pretty clean, huh ? :)
 */
void 
report_tests(hosts)
 struct arglist * hosts;
{
  gtk_widget_show(arg_get_value(MainDialog, "WINDOW"));
  hosts = sort_dangerous_hosts(hosts);
  create_report_window(hosts);
}

/*
 * Opens the report
 */
void open_report(GtkWidget * dontcare, GtkWidget *nsr)
{
struct arglist * arguments = emalloc(sizeof(struct arglist));
file_to_arglist(arguments, gtk_file_selection_get_filename(GTK_FILE_SELECTION(nsr)) );
report_tests(arguments);
}

/*
 * Menu selection to open the report
 */
void open_report_selectfile()
{
 GtkWidget * nsr;
 nsr = gtk_file_selection_new ("Load file");
 /*
  * CWD
  */
 gtk_file_selection_set_filename (GTK_FILE_SELECTION(nsr), ".");
 gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (nsr)->ok_button),
                               "clicked", (GtkSignalFunc) open_report,(void *) nsr );
           
 gtk_signal_connect /*_object*/ (GTK_OBJECT (GTK_FILE_SELECTION
           (nsr)->cancel_button),
             "clicked", (GtkSignalFunc)close_window, (void *)nsr);
           
 gtk_widget_show(nsr);
}

/*
 * Saves the report
 */
static void 
save_report_ask(nul,menu)
  GtkWidget * nul;
  GtkWidget * menu;
{
 GtkWidget * active;
 GtkWidget * filew;
 char * filename;
 char * tmp;
 char * hostname;
 int value;
 char * suffixes[] = {".nsr", ".html", ".txt", ".tex", ""};
 struct arglist * hosts;
 
 active = gtk_menu_get_active(GTK_MENU(menu));
 value = (int)gtk_object_get_data(GTK_OBJECT(active), "type");
 hosts = gtk_object_get_data(GTK_OBJECT(menu), "hosts");
 if(!hosts)
 {
  fprintf(stderr, "Error - NULL hosts in save_report_ask()\n");
  return ;
 }
 if(value < 0)value = 0;
 if(value > SAVE_MAX)value = SAVE_MAX;
 

 hostname = emalloc(strlen(hosts->name)+1);
 strncpy(hostname, hosts->name, strlen(hosts->name));
 
 while((tmp = strchr(hostname, '.')))tmp[0]='_';
 
 filename = emalloc(strlen(hosts->name)+7);
 sprintf(filename, "%s%s", hostname, suffixes[value]);
 efree(&hostname);
 
 filew = gtk_file_selection_new ("Save file");
 gtk_object_set_data(GTK_OBJECT(filew), "type", (void*)value);
 gtk_object_set_data(GTK_OBJECT(filew), "hosts", hosts);
 gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
                               "clicked", (GtkSignalFunc) save_report, filew );
           
 gtk_signal_connect /*_object*/ (GTK_OBJECT (GTK_FILE_SELECTION
           (filew)->cancel_button),
             "clicked", (GtkSignalFunc)close_window, (void *)filew);
 gtk_file_selection_set_filename (GTK_FILE_SELECTION(filew), filename);
           
 gtk_widget_show(filew);
}

/*
 * save_report
 *
 * this function is called when the user
 * clicks on the 'save' item of the file
 * menu...
 */
static void 
save_report(nul,filew)
    GtkWidget * nul;
    GtkWidget * filew;
{
 char * fname = gtk_file_selection_get_filename(GTK_FILE_SELECTION(filew));
 int type = (int)gtk_object_get_data(GTK_OBJECT(filew), "type");
 struct arglist * hosts = gtk_object_get_data(GTK_OBJECT(filew), "hosts");
 
 gtk_widget_hide(filew);
 hosts = sort_dangerous_hosts(hosts);
 switch(type)
 {
  case SAVE_HTML :
  	arglist_to_html(hosts, fname);
	break;
 case SAVE_LATEX :
 	arglist_to_latex(hosts, fname);
	break;
 case SAVE_TEXT :
 	arglist_to_text(hosts, fname);
	break;	

 case SAVE_HTML_GRAPH :
 	arglist_to_html_graph(hosts, fname);
	break;
		
 case SAVE_NSR:
 default :
  	arglist_to_file(hosts, fname);
	break;
 }
 arg_free(hosts);
 gtk_widget_destroy(filew);
}
#endif
